from . import fill_gaps
from . import fpi_sim_matrix_angular
from . import fpi_sim_matrix
from . import fpi_sim
from . import hsti_import
from . import hsti_export
from . import mse
from . import remove_bad_px
from . import remove_vignetting
from . import spectral_debending
from . import voronoi_partitioning

from .fill_gaps import *
from .fpi_sim_matrix_angular import *
from .fpi_sim_matrix import *
from .fpi_sim import *
from .hsti_import import *
from .hsti_export import *
from .mse import *
from .remove_bad_px import *
from .remove_vignetting import *
from .spectral_debending import *
from .voronoi_partitioning import *



import cv2 as cv
import numpy as np
import os

def import_data_cube(path):
    """
    ########## HSTI_import ##########
    This function takes an HSTI numpy array and exports it as individual .ppm
    images to a folder given by folder_name.
    """

    number_of_image_files = sum([ '.ppm' in s for s in os.listdir(path)])

    steps = np.linspace(0,number_of_image_files*10-10,number_of_image_files)

    imgs = np.zeros((768,1024,len(steps)))

    for idx,i in enumerate(steps):
        imgs[:,:,idx] = cv.imread(impath+'step'+str(int(i)) + '.ppm',cv.IMREAD_ANYDEPTH)
    imgs = np.rot90(imgs,1)
    return imgs



import numpy as np
import os
import concurrent.futures as cf

def save_img(img, path):
    with open(path, 'a') as f:
        f.write(f'P3 {img.shape[1]} {img.shape[0]} 255\n')
        for j in range(img.shape[0]):
            for k in range(img.shape[1]):
                f.write(f'{img[j,k]} {img[j,k]} {img[j,k]}\n')
        f.close()

def export_data_cube(data_cube, folder_name):
    """
    ########## HSTI_export ##########
    This function takes an HSTI numpy array and exports it as individual .ppm
    images to a folder given by folder_name.
    """

    os.mkdir(os.getcwd() + '/' + folder_name)
    os.mkdir(os.getcwd() + '/' + folder_name + '/images')
    os.mkdir(os.getcwd() + '/' + folder_name + '/images/capture')
    path = os.getcwd() + '/' + folder_name + '/images/capture'
    data_cube = np.rot90(data_cube, 3)
    img_size = [data_cube.shape[0], data_cube.shape[1]]
    NoB = data_cube.shape[2]

    data_cube_16bit = (65535.9*(data_cube - np.min(data_cube))/(np.max(data_cube) - np.min(data_cube))).astype(np.uint16)

    img_lst, path_lst = [], []
    for i in range(NoB):
        img_lst.append(data_cube_16bit[:,:,i])
        step = i*10
        path_lst.append(f'{path}/step{step}.ppm')

    with cf.ProcessPoolExecutor() as executor:
        results = executor.map(save_img, img_lst, path_lst)



import numpy as np
from scipy.ndimage import median_filter
from IPython.display import clear_output


def baseline(data_cube):
    gray_cube = np.mean(data_cube, axis = 2)
    gray_cube = gray_cube[:,:,np.newaxis]
    gray_cube = np.repeat(gray_cube, data_cube.shape[2], axis = 2)

    data_cube = data_cube - gray_cube

    return data_cube

def standardize(data_cube):
    std_cube = np.std(data_cube, axis = 2)
    std_cube = std_cube[:,:,np.newaxis]
    std_cube = np.repeat(std_cube, data_cube.shape[2], axis = 2)
    baselined_cube = baseline(data_cube)

    data_cube = baselined_cube/std_cube

    return data_cube

def normalize_cube(data_cube):
    min_cube = np.min(data_cube, axis = 2)
    min_cube = min_cube[:,:,np.newaxis]
    min_cube = np.repeat(min_cube, data_cube.shape[2], axis = 2)

    data_cube = data_cube - min_cube
    data_cube = data_cube/np.max(data_cube)

    return data_cube

def normalize_pixel(data_cube):
    min_cube = np.min(data_cube, axis = 2)
    min_cube = min_cube[:,:,np.newaxis]
    min_cube = np.repeat(min_cube, data_cube.shape[2], axis = 2)

    data_cube = data_cube - min_cube

    max_cube = np.max(data_cube, axis = 2)
    max_cube = max_cube[:,:,np.newaxis]
    max_cube = np.repeat(max_cube, data_cube.shape[2], axis = 2)

    data_cube = data_cube/max_cube

    return data_cube

def sbtrct_first_band(data_cube):
    first_cube = data_cube[:,:,0]
    first_cube = first_cube[:,:,np.newaxis]
    first_cube = np.repeat(first_cube, data_cube.shape[2], axis = 2)

    data_cube = data_cube - first_cube

    return data_cube


def median_filter_cube(data_cube, kernel_size, verbose = True):
    NoB = data_cube.shape[2]
    for j in range(NoB):
        if verbose:
            clear_output(wait=True)
            print(f"Median filtering, {round(100*(j+1)/NoB,2)} %")
        data_cube[:,:,j] = median_filter(data_cube[:,:,j], size=kernel_size)

    return data_cube



import numpy as np
from IPython.display import clear_output
import concurrent.futures as cf

def fpi_trans_single_mirror_sep(m_sep, wavelength, temp, mirror_sep_diff, layer_t, fractions):
    transmittance_vec = np.zeros_like(wavelength)
    for idx, wvl in enumerate(wavelength):
        stack_refractive_index = np.array([refrac_Ge(wvl, temp), refrac_ThF4(wvl), refrac_Ge(wvl, temp), 1, refrac_Ge(wvl, temp), refrac_ThF4(wvl), refrac_Ge(wvl, temp)])
        for m_diff, frac in zip(mirror_sep_diff, fractions):
            layer_t[3] = m_sep + m_diff
            T = transfer_FPI(stack_refractive_index, layer_t, wvl)
            transmittance_vec[idx] += frac * abs(1/T[0,0])**2
    return transmittance_vec

#Calculate the difference in mirror separation at radius = r compared to the
#center where r = 0. R is the radius of curvature of the mirror and r is the
#radial distance from the mirror center to the point of evaluation
def mirror_dist_diff(r, R):
    return -2* ( 0.5*(2*R - np.sqrt(4*R**2 - (2*r)**2)) )

def FPI_trans_matrix(mirror_sep, lam, temperature):
    lam0 = 10.5e-6 #Central wavelength used for layer thicknesses
    diameter = 40e-3 #mirror diameter
    h_max = 150e-9 #Maximum height difference between mirror center and edges
    R = h_max/2 + (diameter**2)/(8*h_max) #radius of mirror curvature
    n_points = 9 #number of descrete, different mirror separations across the mirror due to curvature

    r_area = np.linspace(0,diameter/2,n_points) #radii used for area calculations
    r_eval = r_area + np.diff(r_area)[0]/2 #radii at which the mirror separation difference is calculated
    r_area = np.delete(r_area, 0) #Since the first radius is 0, the area is also 0 and is therefore omitted
    r_eval = np.delete(r_eval, -1) #The evaluation points lie in the middle between r_area, but the final r_eval is outside the mirror and is therefore deleted
    area = []
    diff = []
    for i in range(n_points-1):
        diff.append(mirror_dist_diff(r_eval[i],R))
        if i == 0:
            area.append(np.pi*r_area[i]**2) #area of circle
        else:
            area.append(np.pi*(r_area[i]**2-r_area[i-1]**2)) #area of ring
    total_area = sum(area)
    fraction = area/total_area

    transmittance = np.zeros([len(mirror_sep), len(lam)])
    layer_thickness = np.array([0.5*lam0/refrac_Ge(lam0, temperature), 0.25*lam0/refrac_ThF4(lam0),  0.25*lam0/refrac_Ge(lam0, temperature), 0, 0.25*lam0/refrac_Ge(lam0, temperature), 0.25*lam0/refrac_ThF4(lam0), 0.5*lam0/refrac_Ge(lam0, temperature)])

    #Formulate input into lists for later multiprocessing
    lam_lst, temperature_lst, diff_lst, layer_thickness_lst, fraction_lst = [], [], [], [], []
    for i in range(len(mirror_sep)):
        lam_lst.append(lam)
        temperature_lst.append(temperature)
        diff_lst.append(diff)
        layer_thickness_lst.append(layer_thickness)
        fraction_lst.append(fraction)

    with cf.ProcessPoolExecutor() as executor:
        results = executor.map(fpi_trans_single_mirror_sep, mirror_sep, lam_lst, temperature_lst, diff_lst, layer_thickness_lst, fraction_lst)

    for i, result in enumerate(results):
        transmittance[i,:] = result
    return transmittance

def refrac_ThF4(lam):
    # Handbook of Optical Constants of Solids II, 1998, page 1051
    lam = lam * 1e6
    n = 1.118 + 2.46 / lam - 2.98 / (lam**2)
    return n

def refrac_ZnSe(lam):
    # Handbook of Optical Constants of Solids II, page 737
    lam = lam * 1e6
    n = np.sqrt(4 + 1.9*lam**2/(lam**2 - 0.113))
    return n

def refrac_Ge(lam, temperature):
    # Handbook of Optical Constants of Solids I, page 465
    lam = lam * 1e6
    A = -6.04e-3 * temperature + 11.05128
    B = 9.295e-3 * temperature + 4.00536
    C = -5.392e-4 * temperature + 0.599034
    D = 4.151e-4 * temperature + 0.09145
    E = 1.51408 * temperature + 3426.5
    n = np.sqrt(A + (B * lam**2) / (lam**2 - C) + (D * lam**2) / (lam**2 - E))
    return n

def transfer_FPI(n, d, lam):
    T = interface(refrac_ZnSe(lam), n[0]) @ ac_phase(n[0], d[0], lam)
    for i in np.arange(1, len(n), 1):
        T = T @ interface(n[i-1], n[i]) @ ac_phase(n[i], d[i], lam)
    T = T @ interface(n[-1], refrac_ZnSe(lam))
    return T

def interface(n1, n2):
    #n1 is refractive index of leftmost film, while n2 is refracive index of rightmost film
    D = np.ones([2,2])
    D[0, 0] = 1 + n2 / n1
    D[0, 1] = 1 - n2 / n1
    D[1, 0] = 1 - n2 / n1
    D[1, 1] = 1 + n2 / n1
    D = 0.5 * D
    return D

def ac_phase(n, d, lam):
    P = np.array([[0+0j,0+0j], [0+0j,0+0j]])
    P[0,0] = np.exp(1j * n * 2 * np.pi * d / lam)
    P[1, 1] = np.exp(-1j * n * 2 * np.pi * d / lam)
    return P



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="HSTI",
    version="0.0.2",
    author="Mads Nibe et al.",
    author_email="mani@newtec.dk",
    description="The NEWTEC HSTI package contains fundamental functions for the data analysis of hyperspectral thermal images (HSTI).",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    package_data={'': ['data_files/*.txt']},
    package_dir={"": "hsti-analysis"},
    packages=setuptools.find_packages(where="hsti-analysis"),
    python_requires=">=3.6",
)



import numpy as np
import pkg_resources

def remove_vignette(img_cube):

    #Image must have dimensions 1024x768. This function does not calculate
    #vignette based on the image itself. It asumes that the image is taken using
    #a specific camera and applies a predetermined vignetting correction.
    #This function can both be used on single images as well as entire cubes

    resource_package = __name__
    resource_path = '/'.join(('data_files', 'vignetting_profile.txt'))  # Do not use os.path.join()
    path = pkg_resources.resource_stream(resource_package, resource_path)
    vignette = np.loadtxt(path)

    vignette_cube = np.zeros_like(img_cube)

    if len(img_cube.shape) == 3:
        vignette_cube = vignette
        vignette_cube = vignette_cube[:,:,np.newaxis]
        vignette_cube = np.repeat(vignette_cube, img_cube.shape[2], axis = 2)
        devignetted_cube = img_cube/vignette_cube
    elif len(img_cube.shape) == 2:
        devignetted_cube = img_cube/vignette

    return devignetted_cube
